@@ -3,12 +3,12 @@
 # free parameters for each bootstrap sample
 #
 # return COEF matrix of size R x npar (R = number of bootstrap samples)
-# 
+#
 # Ed. 9 mar 2012
 #
 # Notes: - faulty runs are simply ignored (with a warning)
 #        - default R=1000
-# 
+#
 # Updates: - now we have a separate @data slot, we only need to transform once
 #            for the bollen.stine bootstrap (13 dec 2011)
 #          - bug fix: we need to 'update' the fixed.x variances/covariances
@@ -19,10 +19,10 @@
 
 
 
-bootstrapLavaan <- function(object, 
-                            R           = 1000L, 
+bootstrapLavaan <- function(object,
+                            R           = 1000L,
                             type        = "ordinary",
-                            verbose     = FALSE, 
+                            verbose     = FALSE,
                             FUN         = "coef",
                             warn        = -1L,
                             return.boot = FALSE,
@@ -43,7 +43,7 @@ bootstrapLavaan <- function(object,
     # check if options$se is not bootstrap, otherwise, we get an infinite loop
     if(object@Options$se == "bootstrap")
         stop("lavaan ERROR: se == \"bootstrap\"; please refit model with another option for \"se\"")
-     
+
     # check if options$test is not bollen.stine
     if(object@Options$test == "bollen.stine")
         stop("lavaan ERROR: test == \"bollen.stine\"; please refit model with another option for \"test\"")
@@ -53,7 +53,7 @@ bootstrapLavaan <- function(object,
         stop("lavaan ERROR: this function is not (yet) available if conditional.x = TRUE")
     }
 
-    lavoptions. <- list(parallel = parallel, ncpus = ncpus, cl = cl, 
+    lavoptions. <- list(parallel = parallel, ncpus = ncpus, cl = cl,
                         iseed = iseed)
 
     bootstrap.internal(object          = object,
@@ -160,6 +160,10 @@ bootstrap.internal <- function(object          = NULL,
         dataX <- lavdata@X
     }
     dataeXo <- lavdata@eXo
+    dataWT  <- lavdata@weights
+    dataMp  <- lavdata@Mp
+    dataRp  <- lavdata@Rp
+    dataLp  <- lavdata@Lp
 
     # if bollen.stine, transform data here
     if(type == "bollen.stine") {
@@ -184,7 +188,7 @@ bootstrap.internal <- function(object          = NULL,
 
     # if yuan, transform data here
     if(type == "yuan") {
-        # page numbers refer to Yuan et al, 2007      
+        # page numbers refer to Yuan et al, 2007
         # Define a function to find appropriate value of a
         # (p. 272); code supplied 16 jun 2016 by Cheng & Wu
         search.a <- function(F0, d, p) {
@@ -245,7 +249,7 @@ bootstrap.internal <- function(object          = NULL,
             S.inv.sqrt <- lav_matrix_symmetric_sqrt(lavsamplestats@icov[[g]])
 
             X <- lavdata@X[[g]]
-            X <- X %*% S.inv.sqrt %*% S.a.sqrt            
+            X <- X %*% S.inv.sqrt %*% S.a.sqrt
 
             # transformed data
             dataX[[g]] <- X
@@ -261,9 +265,34 @@ bootstrap.internal <- function(object          = NULL,
                 boot.idx <- sample(x=lavsamplestats@nobs[[g]],
                                    size=lavsamplestats@nobs[[g]], replace=TRUE)
                 dataX[[g]] <- dataX[[g]][boot.idx,,drop=FALSE]
-                if(!is.null(dataeXo[[g]]))
+                if(!is.null(dataeXo[[g]])) {
                     dataeXo[[g]] <- dataeXo[[g]][boot.idx,,drop=FALSE]
-            }
+                }
+                if(!is.null(dataWT[[g]])) {
+                    dataWT[[g]] <- dataWT[[g]][boot.idx]
+                }
+                if(lavoptions$missing != "listwise") {
+                    dataMp[[g]] <- lav_data_missing_patterns(dataX[[g]],
+                                       sort.freq = FALSE, coverage = FALSE)
+                }
+                if(length(lavdata@ov.names.x[[g]]) == 0L &&
+                   all(lavdata@ov.names[[g]] %in%
+                       lavdata@ov$name[lavdata@ov$type == "ordered"])) {
+                    dataRp[[g]] <- lav_data_resp_patterns(dataX[[g]])
+                }
+                if(lavdata@nlevels > 1L) {
+                    # extract cluster variable(s), for this group
+                    clus <- matrix(0, nrow(dataX[[g]]), lavdata@nlevels - 1L)
+                    for(l in 2:lavdata@nlevels) {
+                        clus[,(l-1L)] <- lavdata@Lp[[g]]$cluster.idx[[l]]
+                    }
+                    dataLp[[g]] <- lav_data_cluster_patterns(Y = dataX[[g]],
+                                        clus = clus,
+                                        cluster = lavdata@cluster,
+                                        ov.names = lavdata@ov.names[[g]],
+                                        ov.names.l = lavdata@ov.names.l[[g]])
+                }
+            } # g
         } else { # parametric!
             for(g in 1:lavsamplestats@ngroups) {
                 dataX[[g]] <- MASS::mvrnorm(n     = lavsamplestats@nobs[[g]],
@@ -272,16 +301,19 @@ bootstrap.internal <- function(object          = NULL,
             }
         }
 
+        # create new lavdata@ object (new in 0.6-2)
+        newData         <- lavdata
+        newData@X       <- dataX
+        newData@eXo     <- dataeXo
+        newData@weights <- dataWT
+        newData@Mp      <- dataMp
+        newData@Rp      <- dataRp
+        newData@Lp      <- dataLp
+
         # verbose
         if(verbose) cat("  ... bootstrap draw number:", sprintf("%4d", b))
         bootSampleStats <- try(lav_samplestats_from_data(
-                               lavdata       = NULL,
-                               DataX         = dataX,
-                               DataeXo       = dataeXo,
-                               DataOv        = lavdata@ov,
-                               DataOvnames   = lavdata@ov.names,
-                               DataOvnamesx  = lavdata@ov.names.x,
-                               DataWT        = lavdata@weights,
+                               lavdata       = newData,
                                missing       = lavoptions$missing,
                                rescale       = (lavoptions$estimator == "ML" &&
                                                 lavoptions$likelihood == "normal"),
@@ -292,7 +324,7 @@ bootstrap.internal <- function(object          = NULL,
                                group.w.free  = lavoptions$group.w.free,
                                #missing.h1    = (FUN != "coef"), # not if fixed.x, otherwise starting values fails!
                                missing.h1    = TRUE,
-                               verbose       = FALSE), silent=TRUE) 
+                               verbose       = FALSE), silent=TRUE)
         if(inherits(bootSampleStats, "try-error")) {
             if(verbose) {
                 cat("     FAILED: creating sample statistics\n")
@@ -302,17 +334,6 @@ bootstrap.internal <- function(object          = NULL,
             return(NULL)
         }
 
-        # just in case we need the new X in the data slot (lm!)
-        lavdata@X <- dataX
-
-        # adjust model slot if fixed.x variances/covariances
-        # have changed:
-      ### FIXME #####
-        #if(lavmodel@fixed.x && length(vnames(partable, "ov.x")) > 0L) {
-        #    for(g in 1:lavsamplestats@ngroups) {
-        #        
-        #    }
-        #}
         if(lavmodel@fixed.x && length(vnames(lavpartable, "ov.x")) > 0L) {
             model.boot <- NULL
         } else {
@@ -329,17 +350,17 @@ bootstrap.internal <- function(object          = NULL,
             if(verbose) cat("     FAILED: no convergence\n")
             options(old_options)
             return(NULL)
-        } 
-        
+        }
+
         # extract information we need
         if(is.null(object)) { # internal use only!
             if(FUN == "coef") {
                 out <- fit.boot@optim$x
-            } else if(FUN == "test") { 
+            } else if(FUN == "test") {
                 out <- fit.boot@test[[1L]]$stat
-            } else if(FUN == "coeftest") { 
+            } else if(FUN == "coeftest") {
                 out <- c(fit.boot@optim$x, fit.boot@test[[1L]]$stat)
-            } 
+            }
         } else { # general use
             out <- try(FUN(fit.boot, ...), silent=TRUE)
         }
@@ -347,8 +368,8 @@ bootstrap.internal <- function(object          = NULL,
             if(verbose) cat("     FAILED: applying FUN to fit.boot\n")
             options(old_options)
             return(NULL)
-        } 
-        if(verbose) cat("   OK -- niter = ", 
+        }
+        if(verbose) cat("   OK -- niter = ",
                         sprintf("%3d", fit.boot@optim$iterations), " fx = ",
                         sprintf("%13.9f", fit.boot@optim$fx), "\n")
         out
@@ -399,7 +420,7 @@ bootstrap.internal <- function(object          = NULL,
     # NOT DONE YET
     if(return.boot) {
         # mimic output boot function
-    } 
+    }
 
     # restore options
     options(old_options)
