@@ -81,7 +81,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             dotdotdot$optim.gradient <- dotdotdot$gradient
         }
         # init_nelder_mead
-        if(!!is.null(dotdotdot$control$init_nelder_mead)) {
+        if(!is.null(dotdotdot$control$init_nelder_mead)) {
             dotdotdot$optim.init_nelder_mead <-
                 dotdotdot$control$init_nelder_mead
         }
@@ -220,7 +220,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             stop("lavaan ERROR: when data is clustered, you must specify a model\n", "  for each level in the model syntax (for now); see example(Demo.twolevel)")
         }
 
-        tmp.lav <- lavaanify(FLAT, ngroups = tmp.ngroups, warn = FALSE) 
+        tmp.lav <- lavaanify(FLAT, ngroups = tmp.ngroups, warn = FALSE)
         # check for empty levels
         if(max(tmp.lav$level) < 2L) {
             stop("lavaan ERROR: at least one level has no model syntax; you must specify a model for each level in the model syntax (for now); see example(Demo.twolevel)")
@@ -231,7 +231,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                 ov.names.l[[g]] <- vector("list", length = tmp.nlevels)
             for(l in seq_len(tmp.nlevels)) {
                 if(tmp.ngroups > 1L) {
-                    ov.names.l[[g]][[l]] <- 
+                    ov.names.l[[g]][[l]] <-
                     unique(unlist(lav_partable_vnames(tmp.lav,
                                                   type = "ov",
                                                   group = tmp.group.values[g],
@@ -257,14 +257,12 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             ngroups <- lav_partable_ngroups(FLAT)
             ov.names.l <- vector("list", length = ngroups)
             for(g in 1:ngroups) {
+                # note: lavNames() will return a list if any level:
                 ov.names.l[[g]] <- lavNames(FLAT, "ov", group = g)
             }
         } else {
             # no level: in model syntax
             ov.names.l <- list()
-            if(length(cluster) > 0L) {
-                stop("lavaan ERROR: when data is clustered, you must specify a model\n", "  for each level in the model syntax (for now); see example(Demo.twolevel)") 
-            }
         }
     }
 
@@ -273,9 +271,6 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
 
 
 
-
-
-
     #######################
     #### 2. lavoptions ####
     #######################
@@ -287,7 +282,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         if(length(dotdotdot) > 0L) {
             dot.names <- names(dotdotdot)
             op.idx <- which(dot.names %in% names(slotOptions))
-            warning("lavaan WARNING: the following argument(s) override(s) the options in slotOptions:\n\t\t", paste(dot.names[op.idx], collapse = " ")) 
+            warning("lavaan WARNING: the following argument(s) override(s) the options in slotOptions:\n\t\t", paste(dot.names[op.idx], collapse = " "))
             lavoptions[ dot.names[op.idx] ] <- dotdotdot[ op.idx ]
         }
     } else {
@@ -326,8 +321,15 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             opt$categorical <- FALSE
         }
 
+        # clustered?
+        if(length(cluster) > 0L) {
+            opt$clustered <- TRUE
+        } else {
+            opt$clustered <- FALSE
+        }
+
         # multilevel?
-        if(length(ov.names.l) > 0L) {
+        if(length(ov.names.l) > 0L && length(ov.names.l[[1]]) > 1L) {
             opt$multilevel <- TRUE
         } else {
             opt$multilevel <- FALSE
@@ -457,7 +459,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     ########################
     if(!is.null(slotParTable)) {
         lavpartable <- slotParTable
-    } else if(is.character(model) || 
+    } else if(is.character(model) ||
               inherits(model, "formula")) {
         # check FLAT before we proceed
         if(lavoptions$debug) {
@@ -536,7 +538,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                               lavpartable$free == 0L &
                               lavpartable$ustart == 0)
         if(length(zero.var.idx) > 0L) {
-            lavpartable$ustart[zero.var.idx] <- lavoptions$em.zerovar.offset    
+            lavpartable$ustart[zero.var.idx] <- lavoptions$em.zerovar.offset
         }
     }
 
@@ -628,7 +630,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                 h1.implied      <- out$implied
                 h1.loglik       <- out$logl$loglik
                 h1.loglik.group <- out$logl$loglik.group
-    
+
                 # collect in h1 list
                 lavh1 <- list(implied      = h1.implied,
                               loglik       = h1.loglik,
@@ -696,7 +698,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                                debug          = lavoptions$debug)
 
             # sanity check
-            if("start" %in% lavoptions$check) {
+            if(!is.null(lavoptions$check.start) && lavoptions$check.start) {
                 START <- lav_start_check_cov(lavpartable = lavpartable,
                                              start       = START)
             }
@@ -731,18 +733,18 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             # check/warn if theta/delta values make sense
             if(!all(lavpartable$start == lavpartable$ustart)) {
                 if(lavmodel@parameterization == "delta") {
-                    # did the user specify theta values? 
+                    # did the user specify theta values?
                     user.var.idx <- which(lavpartable$op == "~~" &
                            lavpartable$lhs == lavpartable$rhs &
                            lavpartable$lhs %in% unlist(lavpta$vnames$ov.ord) &
                            lavpartable$user == 1L)
                     if(length(user.var.idx)) {
-                        warning("lavaan WARNING: ", 
-              "variance (theta) values for categorical variables are ignored", 
+                        warning("lavaan WARNING: ",
+              "variance (theta) values for categorical variables are ignored",
               "\n\t\t  if parameterization = \"delta\"!")
                     }
                 } else if(lavmodel@parameterization == "theta") {
-                    # did the user specify theta values? 
+                    # did the user specify theta values?
                     user.delta.idx <- which(lavpartable$op == "~*~" &
                             lavpartable$lhs == lavpartable$rhs &
                             lavpartable$lhs %in% unlist(lavpta$vnames$ov.ord) &
@@ -779,9 +781,9 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
         ov.types <- lavdata@ov$type
         if(lavmodel@conditional.x && lavmodel@nexo > 0L) {
             # remove ov.x
-            ov.x.idx <- unlist(lavpta$vidx$ov.x)   
+            ov.x.idx <- unlist(lavpta$vidx$ov.x)
             ov.types <- ov.types[-ov.x.idx]
-        } 
+        }
 
         if(lavoptions$estimator == "PML" && all(ov.types == "ordered")) {
             TH <- computeTH(lavmodel)
@@ -971,13 +973,14 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
                                lavimplied     = NULL,
                                lavpartable    = lavpartable,
                                lavmodel       = lavmodel,
-                               lavoptions     = lavoptions, 
+                               lavoptions     = lavoptions,
                                verbose        = lavoptions$verbose,
                                fx.tol         = lavoptions$em.fx.tol,
                                dx.tol         = lavoptions$em.dx.tol,
                                max.iter       = lavoptions$em.iter.max)
         } else {
             x <- lav_model_estimate(lavmodel        = lavmodel,
+                                    lavpartable     = lavpartable,
                                     lavsamplestats  = lavsamplestats,
                                     lavdata         = lavdata,
                                     lavoptions      = lavoptions,
@@ -994,7 +997,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
             lavmodel@con.lambda <- attr(x, "con.lambda")
         # check if model has converged or not
         if(!attr(x, "converged") && lavoptions$warn) {
-           warning("lavaan WARNING: model has NOT converged!")
+           warning("lavaan WARNING: the optimizer warns that a solution has NOT been found!")
         }
     } else {
         x <- numeric(0L)
@@ -1021,6 +1024,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     lavoptim$npar <- length(x)
     lavoptim$iterations <- attr(x, "iterations")
     lavoptim$converged  <- attr(x, "converged")
+    lavoptim$parscale   <- attr(x, "parscale")
     fx.copy <- fx <- attr(x, "fx"); attributes(fx) <- NULL
     lavoptim$fx         <- fx
     lavoptim$fx.group   <- attr(fx.copy, "fx.group")
@@ -1049,8 +1053,8 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     lavimplied <- list()
     if(lavoptions$implied) {
          lavimplied <- lav_model_implied(lavmodel)
-    } 
-        
+    }
+
     lavloglik <- list()
     if(lavoptions$loglik) {
          lavloglik <- lav_model_loglik(lavdata        = lavdata,
@@ -1063,7 +1067,7 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
     timing$implied <- (proc.time()[3] - start.time)
     start.time <- proc.time()[3]
 
-    
+
 
 
 
@@ -1231,11 +1235,32 @@ lavaan <- function(# user-specified model: can be syntax, parameter Table, ...
 
 
 
-    # post-fitting check
-    if("post" %in% lavoptions$check && lavTech(lavaan, "converged")) {
+    # post-fitting check of parameters
+    if(!is.null(lavoptions$check.post) && lavoptions$check.post &&
+       lavTech(lavaan, "converged")) {
         lavInspect(lavaan, "post.check")
     }
 
+    # new in 0.6-2
+    # FIXME: not scale independent (should use solve(Hessian) %*% g)
+    # but Hessian is not always available (or expensive to compute)
+    hasExplicitConstraints <- FALSE
+    if(is.character(constraints) && nchar(constraints) > 0L) {
+        hasExplicitConstraints <- TRUE
+    }
+    if(!is.null(lavoptions$check.gradient) &&
+       lavoptions$check.gradient && lavTech(lavaan, "converged") &&
+       !hasExplicitConstraints) {
+        grad <- lavInspect(lavaan, "optim.gradient")
+        large.idx <- which(abs(grad) > 0.001)  # better 0.0001?
+        if(length(large.idx) > 0L) {
+            warning(
+  "lavaan WARNING: not all elements of the gradient are (near) zero;\n",
+"                  the optimizer may not have found a local solution;\n",
+"                  use lavInspect(fit, \"optim.gradient\") to investigate")
+        }
+    }
+
     ########################
     #### post: baseline ####
     ########################
